@@ -1,3 +1,4 @@
+
 outliers.depth.trim<-function(fdataobj,nb=200,smo=0.05,trim=0.01,
 dfunc=depth.mode,...){
  if (!is.fdata(fdataobj)) fdataobj=fdata(fdataobj)
@@ -39,7 +40,7 @@ dfunc=depth.mode,...){
             if (modal) {
              mdist<-dd$dist[-elim,-elim]
             class(mdist)<-c("matrix","fdist")        
-            dd<-dfunc(curvasgood,trim=trim,metric=mdist,scale=FALSE,...)
+            dd<-dfunc(curvasgood,trim=trim,metric=mdist,scale=FALSE)
             }
           else dd<-dfunc(curvasgood,trim=trim,...)
           d<-dd$dep 
