@@ -4,9 +4,12 @@ x<-fdataobj[["data"]]
 tt<-fdataobj[["argvals"]]
 rtt<-fdataobj[["rangeval"]]
 n<-nrow(fdataobj)
+rowname.ori<-row.names(fdataobj[["data"]])
+rowname.num<-1:n
+row.names(fdataobj[["data"]])<-rowname.num    
     m<-ncol(fdataobj)
     if (is.null(n) && is.null(m)) stop("Error in in the data dimensions")
-    out.thres.lrt<-outliers.thres.lrt(x,nb=nb,smo=smo,trim=trim,...)
+    out.thres.lrt<-outliers.thres.lrt(fdataobj,nb=nb,smo=smo,trim=trim,...)
     hay<-1
     outliers<-c()
     valor.estadistico<-c()
@@ -18,26 +21,29 @@ n<-nrow(fdataobj)
     while (hay==1 && i<maxiter){
           i<-i+1
           aux<-c()
-          auxmean<-func.trim.mode(curvasgood,trim=trim,...)[["data"]][1,]
+          auxmean<-func.trim.mode(curvasgood,trim=trim,...)#[["data"]][1,]
           aa<-func.trimvar.mode(curvasgood,trim=trim,...)
-          auxdt<-sqrt(aa[["data"]][1,])
+          auxdt<-sqrt(aa)
+          divi<-auxmean/sqrt(aa)    
           for (j in 1:ngood){
-              aux[j]<-metric.lp(curvasgood[["data"]][j,]/auxdt,auxmean/auxdt,...)}
+#cat(j);          print(metric.lp(curvasgood[j]/auxdt,divi,...))
+              aux[j]<-metric.lp(curvasgood[j]/auxdt,divi,...)[1,1]
+              }
           maximo<-as.numeric(max(aux))
-          fecha<-as.numeric(row.names(curvasgood[["data"]])[maximo==aux])
+          fecha<-rowname.num[maximo==aux] 
           elim<-which(maximo==aux)
           if (maximo>out.thres.lrt){
-             curvasgood<-curvasgood[elim,]
-             outliers<-c(outliers,fecha)
+             curvasgood<-curvasgood[-elim]
+             outliers<-c(outliers,rowname.ori[fecha])
              valor.estadistico<-c(valor.estadistico,maximo)
              nout<-nout+1
              ngood<-n-nout
           }
           if (maximo<out.thres.lrt){hay<-0}
+          
     }
 if (i==maxiter) print("convergencia no lograda")
 return(list("outliers"=outliers,"stat.value"=valor.estadistico,
 "percentile"=out.thres.lrt))
 c(outliers,valor.estadistico,out.thres.lrt)
 }
-
